<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

use Conto\Quickpay\OrderService;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

class Quickpay extends PaymentModule
{
    private $_html = '';
    private $_postErrors = [];

    private $secretKey;
    private $salt;

    public function __construct()
    {
        $this->name = 'quickpay';
        $this->tab = 'payments_gateways';
        $this->version = '0.1.0';
        $this->author = 'Conto QuickPay';

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $configs = $this->getConfigFieldsValues();

        if (isset($configs['QUICKPAY_SECRET_KEY'])) {
            $this->secretKey = $configs['QUICKPAY_SECRET_KEY'];
        }
        if (isset($configs['QUICKPAY_CONTRACT_ID'])) {
            $this->salt = $configs['QUICKPAY_CONTRACT_ID'];
        }

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Conto QuickPay');
        $this->description = $this->l('Allows e-shops to receive their payments online from buyers, who have a bank account but do not want to use a payment card.');
        $this->confirmUninstall = $this->l('Are you sure you want to delete these details?');
        $this->ps_versions_compliancy = ['min' => '1.7.0.0', 'max' => _PS_VERSION_];

        if (!isset($this->secretKey) || empty($this->salt)) {
            $this->warning = $this->l('The "Secret key", "Salt", fields must be configured before using this module.');
        }

        if (!count(CurrencyCore::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    public function install()
    {
        Configuration::updateValue('QUICKPAY_MIN_ORDER_AMOUNT', 0.01);

        return parent::install()
            && $this->registerHook('moduleRoutes')
            && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn');
        ;
    }

    public function uninstall()
    {
        return Configuration::deleteByName('QUICKPAY_SECRET_KEY')
            && Configuration::deleteByName('QUICKPAY_CONTRACT_ID')
            && Configuration::deleteByName('QUICKPAY_MIN_ORDER_AMOUNT')
            && Configuration::deleteByName('QUICKPAY_HOST')
            && Configuration::deleteByName('QUICKPAY_CONFIRMED_ID')
            && Configuration::deleteByName('QUICKPAY_FAILED_ID')
            && Configuration::deleteByName('QUICKPAY_EXPIRED_ID')
            && Configuration::deleteByName('QUICKPAY_SIGNED_ID')
            && Configuration::deleteByName('QUICKPAY_PENDING_ID')
            && Configuration::deleteByName('QUICKPAY_PROCESSING_ID')
            && $this->unregisterHook('moduleRoutes')
            && $this->unregisterHook('paymentOptions' )
            && $this->unregisterHook('paymentReturn')
            && parent::uninstall()
            ;
    }

    private function _postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('QUICKPAY_SECRET_KEY')) {
                $this->_postErrors[] = $this->l('The "Secret key" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_CONTRACT_ID')) {
                $this->_postErrors[] = $this->l('The "Contract ID" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_CONFIRMED_ID')) {
                $this->_postErrors[] = $this->l('The "CONFIRMED" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_EXPIRED_ID')) {
                $this->_postErrors[] = $this->l('The "EXPIRED" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_FAILED_ID')) {
                $this->_postErrors[] = $this->l('The "FAILED" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_SIGNED_ID')) {
                $this->_postErrors[] = $this->l('The "SIGNED" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_PENDING_ID')) {
                $this->_postErrors[] = $this->l('The "PENDING" field is required.');
            }
            if (!Tools::getValue('QUICKPAY_PROCESSING_ID')) {
                $this->_postErrors[] = $this->l('The "PENDING" field is required.');
            }
            if (
                !empty(Tools::getValue('QUICKPAY_MIN_ORDER_AMOUNT'))
                && (float) Tools::getValue('QUICKPAY_MIN_ORDER_AMOUNT') < 0.01
            ) {
                $this->_postErrors[] = $this->l('The "Minimal order amount" should be higher or equal to 0.01 euro');
            }
        }
    }

    private function _postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue(
                'QUICKPAY_SECRET_KEY',
                Tools::getValue('QUICKPAY_SECRET_KEY')
            );
            Configuration::updateValue(
                'QUICKPAY_CONTRACT_ID',
                Tools::getValue('QUICKPAY_CONTRACT_ID')
            );
            if (empty(Tools::getValue('QUICKPAY_MIN_ORDER_AMOUNT'))) {
                Configuration::updateValue(
                    'QUICKPAY_MIN_ORDER_AMOUNT',
                    0.01
                );
            } else {
                Configuration::updateValue(
                    'QUICKPAY_MIN_ORDER_AMOUNT',
                    number_format(
                        (float)Tools::getValue('QUICKPAY_MIN_ORDER_AMOUNT'),
                        2,
                        '.',
                        ''
                    )
                );
            }
            Configuration::updateValue(
                'QUICKPAY_HOST',
                Tools::getValue('QUICKPAY_HOST')
            );
            Configuration::updateValue(
                'QUICKPAY_EXPIRED_ID',
                Tools::getValue('QUICKPAY_EXPIRED_ID')
            );
            Configuration::updateValue(
                'QUICKPAY_CONFIRMED_ID',
                Tools::getValue('QUICKPAY_CONFIRMED_ID')
            );
            Configuration::updateValue(
                'QUICKPAY_FAILED_ID',
                Tools::getValue('QUICKPAY_FAILED_ID')
            );
            Configuration::updateValue(
                'QUICKPAY_SIGNED_ID',
                Tools::getValue('QUICKPAY_SIGNED_ID')
            );
            Configuration::updateValue(
                'QUICKPAY_PENDING_ID',
                Tools::getValue('QUICKPAY_PENDING_ID')
            );
            Configuration::updateValue(
                'QUICKPAY_PROCESSING_ID',
                Tools::getValue('QUICKPAY_PROCESSING_ID')
            );
        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    public function getContent()
    {
        $this->_html = '';

        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        }

        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    public function hookPaymentOptions($params)
    {
        /** @var CartCore $cart */
        $cart = $params['cart'];

        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        if (!$this->isOrderTotalIsHigherThanMinOrderAmount($cart)) {
            return;
        }

        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->l('Payment via internet bank'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', [], true))
            ->setLogo(Media::getMediaPath(
                _PS_MODULE_DIR_ . $this->name . '/views/images/quickpay_logo.png'
            ));


        return [$newOption];
    }

    public function hookModuleRoutes($params)
    {

        $my_routes = array(
            'module-quickpay-callback' => array(
                'controller' => 'callback',
                'rule' => 'quickpay/callback',
                'keywords' => array(
                    'module' => ['regexp' => '[_a-zA-Z0-9_-]+', 'param' => 'module'],
                    'controller' => ['regexp' => '[_a-zA-Z0-9_-]+', 'param' => 'controller'],
                ),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'quickpay',
                    'controller' => 'callback'
                ),
            ),
            'module-quickpay-redirect' => array(
                'controller' => 'redirect',
                'rule' => 'quickpay/redirect',
                'keywords' => array(
                    'module' => ['regexp' => '[_a-zA-Z0-9_-]+', 'param' => 'module'],
                    'controller' => ['regexp' => '[_a-zA-Z0-9_-]+', 'param' => 'controller'],
                ),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'quickpay',
                    'controller' => 'redirect'
                ),
            )
        );

        return $my_routes;
    }

    public function hookDisplayPaymentReturn($params)
    {
        if (!$this->active) {
            return false;
        }

        return $this->display(__FILE__, 'payment_return.tpl');
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency((int) ($cart->id_currency));
        $currencies_module = $this->getCurrency((int) $cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency'] && $currency_order->iso_code == 'EUR') {
                    return true;
                }
            }
        }

        return false;
    }

    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Details of Conto QuickPay module')
                ],
                'tabs' => [
                    'credentials' => $this->l('Credentials'),
                    'states' => $this->l('Order statuses'),
                    'other' => $this->l('Other')

                ],
                'input' => [
                    [
                        'type' => 'text',
                        'tab' => 'credentials',
                        'label' => $this->l('Secret key'),
                        'name' => 'QUICKPAY_SECRET_KEY',
                        'required' => true,
                        'desc' => $this->l('Provided by Conto QuickPay')
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'credentials',
                        'label' => $this->l('Contract id'),
                        'name' => 'QUICKPAY_CONTRACT_ID',
                        'required' => true,
                        'desc' => $this->l('Provided by Conto QuickPay')
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'credentials',
                        'label' => $this->l('Conto QuickPay widget host'),
                        'name' => 'QUICKPAY_HOST',
                        'desc' => $this->l('Add host without / of the end(e.g. https://quickpay.contomobile.com). If you do not have this value, please leave empty.')
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'other',
                        'label' => $this->l('Minimum order amount'),
                        'name' => 'QUICKPAY_MIN_ORDER_AMOUNT',
                        'desc' => $this->l('Minimal order amount. The default value is 0.01 Euro')
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'states',
                        'label' => $this->l('Confirmed'),
                        'name' => 'QUICKPAY_CONFIRMED_ID',
                        'required' => true,
                        'options' => [
                            'query' => OrderService::getOrderStatuses(),
                            'id' => 'order_state_id',
                            'name' => 'name'
                        ],
                        'desc' => $this->l('Choose order status for Confirmed payment transaction')
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'states',
                        'label' => $this->l('Failed'),
                        'name' => 'QUICKPAY_FAILED_ID',
                        'required' => true,
                        'options' => [
                            'query' => OrderService::getOrderStatuses(),
                            'id' => 'order_state_id',
                            'name' => 'name'
                        ],
                        'desc' => $this->l('Choose order status for Failed payment transaction')
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'states',
                        'label' => $this->l('Expired'),
                        'name' => 'QUICKPAY_EXPIRED_ID',
                        'required' => true,
                        'options' => [
                            'query' => OrderService::getOrderStatuses(),
                            'id' => 'order_state_id',
                            'name' => 'name'
                        ],
                        'desc' => $this->l('Choose order status for Expired payment transaction')
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'states',
                        'label' => $this->l('Signed'),
                        'name' => 'QUICKPAY_SIGNED_ID',
                        'required' => true,
                        'options' => [
                            'query' => OrderService::getOrderStatuses(),
                            'id' => 'order_state_id',
                            'name' => 'name'
                        ],
                        'desc' => $this->l('Choose order status for Signed payment transaction')
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'states',
                        'label' => $this->l('Pending'),
                        'name' => 'QUICKPAY_PENDING_ID',
                        'required' => true,
                        'options' => [
                            'query' => OrderService::getOrderStatuses(),
                            'id' => 'order_state_id',
                            'name' => 'name'
                        ],
                        'desc' => $this->l('Choose order status for Pending payment transaction')
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'states',
                        'label' => $this->l('Processing'),
                        'name' => 'QUICKPAY_PROCESSING_ID',
                        'required' => true,
                        'options' => [
                            'query' => OrderService::getOrderStatuses(),
                            'id' => 'order_state_id',
                            'name' => 'name'
                        ],
                        'desc' => $this->l('Choose order status for Processing payment transaction')
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];

        $helper = new HelperFormCore();
        $helper->show_toolbar = false;
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
        ];

        return $helper->generateForm([$fields_form]);
    }

    private function isOrderTotalIsHigherThanMinOrderAmount($cart)
    {
        return $cart->getOrderTotal() >= Configuration::get('QUICKPAY_MIN_ORDER_AMOUNT');
    }

    private function getOrderStatuses()
    {
        $orderStates = OrderStateCore::getOrderStates(1);

        $orderStatuses = [];

        foreach ($orderStates as $orderState) {
            $orderStatuses[] = [
                'order_state_id' => $orderState['id_order_state'],
                'name' => $orderState['name']
            ];
        }

        return $orderStatuses;
    }

    private function getConfigFieldsValues()
    {
        return Configuration::getMultiple(
            [
                'QUICKPAY_SECRET_KEY',
                'QUICKPAY_CONTRACT_ID',
                'QUICKPAY_MIN_ORDER_AMOUNT',
                'QUICKPAY_HOST',
                'QUICKPAY_CONFIRMED_ID',
                'QUICKPAY_FAILED_ID',
                'QUICKPAY_EXPIRED_ID',
                'QUICKPAY_SIGNED_ID',
                'QUICKPAY_PENDING_ID',
                'QUICKPAY_PROCESSING_ID',
            ]
        );
    }
}
